@@ -30,7 +30,7 @@ class TemplatedNaturalLanguageGenerator(NaturalLanguageGenerator):
         self, filled_slots: Dict[Text, Any], response: Dict[Text, Any]
     ) -> bool:
         """Checks if the conditional response variation matches the filled slots."""
-        constraints = response.get(RESPONSE_CONDITION)
+        constraints = response.get(RESPONSE_CONDITION, [])
         for constraint in constraints:
             name = constraint["name"]
             value = constraint["value"]
