@@ -13,6 +13,7 @@ from http import HTTPStatus
 from typing import (
     Any,
     Callable,
+    DefaultDict,
     List,
     Optional,
     Text,
@@ -37,6 +38,8 @@ from rasa.nlu.emulators.emulator import Emulator
 import rasa.utils.common
 import rasa.shared.utils.common
 import rasa.shared.utils.io
+import rasa.shared.utils.validation
+import rasa.shared.nlu.training_data.schemas.data_schema
 import rasa.utils.endpoints
 import rasa.utils.io
 from rasa.shared.core.training_data.story_writer.yaml_story_writer import (
@@ -63,7 +66,10 @@ from rasa.core.channels.channel import (
 import rasa.shared.core.events
 from rasa.shared.core.events import Event
 from rasa.core.test import test
-from rasa.shared.core.trackers import DialogueStateTracker, EventVerbosity
+from rasa.shared.core.trackers import (
+    DialogueStateTracker,
+    EventVerbosity,
+)
 from rasa.core.utils import AvailableEndpoints
 from rasa.nlu.emulators.no_emulator import NoEmulator
 import rasa.nlu.test
@@ -81,7 +87,7 @@ if TYPE_CHECKING:
     ]
     SanicView = Callable[
         [Arg(Request, "request"), VarArg(), KwArg()],  # noqa: F821
-        SanicResponse,
+        Coroutine[Any, Any, SanicResponse],
     ]
 
 
@@ -168,10 +174,12 @@ def ensure_conversation_exists() -> Callable[["SanicView"], "SanicView"]:
 
     def decorator(f: "SanicView") -> "SanicView":
         @wraps(f)
-        def decorated(request: Request, *args: Any, **kwargs: Any) -> "SanicResponse":
+        async def decorated(
+            request: Request, *args: Any, **kwargs: Any
+        ) -> "SanicResponse":
             conversation_id = kwargs["conversation_id"]
-            if request.app.ctx.agent.tracker_store.exists(conversation_id):
-                return f(request, *args, **kwargs)
+            if await request.app.ctx.agent.tracker_store.exists(conversation_id):
+                return await f(request, *args, **kwargs)
             else:
                 raise ErrorResponse(
                     HTTPStatus.NOT_FOUND, "Not found", "Conversation ID not found."
@@ -288,7 +296,7 @@ def event_verbosity_parameter(
         )
 
 
-def get_test_stories(
+async def get_test_stories(
     processor: "MessageProcessor",
     conversation_id: Text,
     until_time: Optional[float],
@@ -308,9 +316,11 @@ def get_test_stories(
         The stories for `conversation_id` in test format.
     """
     if fetch_all_sessions:
-        trackers = processor.get_trackers_for_all_conversation_sessions(conversation_id)
+        trackers = await processor.get_trackers_for_all_conversation_sessions(
+            conversation_id
+        )
     else:
-        trackers = [processor.get_tracker(conversation_id)]
+        trackers = [await processor.get_tracker(conversation_id)]
 
     if until_time is not None:
         trackers = [tracker.travel_back_in_time(until_time) for tracker in trackers]
@@ -355,7 +365,7 @@ async def update_conversation_with_events(
         The tracker for `conversation_id` with the updated events.
     """
     if rasa.shared.core.events.do_events_begin_with_session_start(events):
-        tracker = processor.get_tracker(conversation_id)
+        tracker = await processor.get_tracker(conversation_id)
     else:
         tracker = await processor.fetch_tracker_with_initial_session(conversation_id)
 
@@ -755,9 +765,7 @@ def create_app(
                     await processor.execute_side_effects(
                         events, tracker, output_channel
                     )
-
-                app.ctx.agent.tracker_store.save(tracker)
-
+                await app.ctx.agent.tracker_store.save(tracker)
             return response.json(tracker.current_state(verbosity))
         except Exception as e:
             logger.debug(traceback.format_exc())
@@ -806,7 +814,7 @@ def create_app(
                 )
 
                 # will override an existing tracker with the same id!
-                app.ctx.agent.tracker_store.save(tracker)
+                await app.ctx.agent.tracker_store.save(tracker)
 
             return response.json(tracker.current_state(verbosity))
         except Exception as e:
@@ -829,7 +837,7 @@ def create_app(
         )
 
         try:
-            stories = get_test_stories(
+            stories = await get_test_stories(
                 app.ctx.agent.processor,
                 conversation_id,
                 until_time,
@@ -1022,7 +1030,7 @@ def create_app(
                 tracker = await app.ctx.agent.processor.run_action_extract_slots(
                     user_message.output_channel, tracker
                 )
-                app.ctx.agent.processor.save_tracker(tracker)
+                await app.ctx.agent.processor.save_tracker(tracker)
 
             return response.json(tracker.current_state(verbosity))
         except Exception as e:
@@ -1149,7 +1157,9 @@ def create_app(
                     test_data, config_file, int(cross_validation_folds)
                 )
         else:
-            test_data = _test_data_file_from_payload(request, temporary_directory)
+            payload = _nlu_training_payload_from_json(request, temporary_directory)
+            test_data = payload.get("training_files")
+
             if cross_validation_folds:
                 raise ErrorResponse(
                     HTTPStatus.BAD_REQUEST,
@@ -1163,7 +1173,8 @@ def create_app(
             )
 
         try:
-            evaluation = await test_coroutine
+            if test_coroutine is not None:
+                evaluation = await test_coroutine
             return response.json(evaluation)
         except Exception as e:
             logger.error(traceback.format_exc())
@@ -1236,7 +1247,7 @@ def create_app(
             "response_selection_evaluation": response_selector_report,
         }
 
-        result: defaultdict[Text, Any] = defaultdict(dict)
+        result: DefaultDict[Text, Any] = defaultdict(dict)
         for evaluation_name, evaluation in eval_name_mapping.items():
             report = evaluation.evaluation.get("report", {})
             averages = report.get("weighted avg", {})
@@ -1468,6 +1479,33 @@ def _training_payload_from_yaml(
     )
 
 
+def _nlu_training_payload_from_json(
+    request: Request, temp_dir: Path, file_name: Text = "data.json"
+) -> Dict[Text, Any]:
+    logger.debug("Extracting JSON training data from request body.")
+
+    rasa.shared.utils.validation.validate_training_data(
+        request.json,
+        rasa.shared.nlu.training_data.schemas.data_schema.rasa_nlu_data_schema(),
+    )
+    training_data = temp_dir / file_name
+    rasa.shared.utils.io.dump_obj_as_json_to_file(training_data, request.json)
+
+    model_output_directory = str(temp_dir)
+    if rasa.utils.endpoints.bool_arg(request, "save_to_default_model_directory", True):
+        model_output_directory = DEFAULT_MODELS_PATH
+
+    return dict(
+        domain=str(training_data),
+        config=str(training_data),
+        training_files=str(temp_dir),
+        output=model_output_directory,
+        force_training=rasa.utils.endpoints.bool_arg(request, "force_training", False),
+        core_additional_arguments=_extract_core_additional_arguments(request),
+        nlu_additional_arguments=_extract_nlu_additional_arguments(request),
+    )
+
+
 def _validate_yaml_training_payload(yaml_text: Text) -> None:
     try:
         RasaYAMLReader().validate(yaml_text)
